@@ -20,9 +20,9 @@ from coordio.defaults import IHAT, JHAT, KHAT, calibration
 import pandas as pd
 
 
-positionerTable = calibration.positionerTable
-wokCoords = calibration.wokCoords
-fiducialCoords = calibration.fiducialCoords
+# positionerTable = calibration.positionerTable
+# wokCoords = calibration.wokCoords
+# fiducialCoords = calibration.fiducialCoords
 
 # __all__ = ['RobotGrid', 'RobotGridFilledHex']
 # default orientation of positioner to wok
@@ -49,10 +49,19 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
     ----------
 
     stepSize : float, np.float32
-        step size for paths (degrees), default 1.
+        step size for paths (degrees), default 0.1.
+
+    epsilon : float, or None
+        distance metric for path simplification, smaller
+        values yeild more points, default is stepsize * 2.2
+
+    seed : int
+        random seed for grid
+
+    scaleFac : float
+        multiplicative scale factor to model thermal expansion of wok,
+        default is 1
 
-    collisionBuffer : float, np.float32
-        half-width of beta arm, including buffer (mm), default 2.0
 
     Attributes:
     ----------
@@ -61,9 +70,6 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         step size for paths (degrees). A maximum perturbation allowed for either
         alpha or beta axes
 
-    collisionBuffer : float, np.float32
-        half-width of beta arm, including buffer (mm)
-
     epsilon : float
         smoothing parameter used in the RDP path simplification.  Smaller values
         mean smaller deviations from the rough path are allowed.
@@ -91,18 +97,17 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         path generation failed, not all robots reached target
 
     nSteps : int
-        steps taken to
+        steps taken to generate a path
 
     """
     def __init__(
-        self, stepSize=1., collisionBuffer=2.0,
+        self, stepSize=0.1,
         epsilon=None, seed=0, scaleFac=1):
 
-        np.random.seed(0) # for seedifying stuff out here in python
+        np.random.seed(seed) # for seedifying stuff out here in python
 
         self.stepSize = stepSize
         self.scaleFac = scaleFac
-        self.collisionBuffer = collisionBuffer
         if epsilon is None:
             self.epsilon = stepSize * 2.2
         else:
@@ -110,8 +115,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         self.seed = seed
         self.totalReplaced = 0 # hack for now until it goes into the C++
         self.runtime = 0 # hack for now until
-        super().__init__(self.stepSize, self.collisionBuffer,
-                         self.epsilon, self.seed)
+        super().__init__(self.stepSize, self.epsilon, self.seed)
         self._load_grid()
         return
 
@@ -163,47 +167,56 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
             self.decollideGrid()
         return self.getPathPair()
 
-    def getPathPair(self, speed=2):
+    def getPathPair(
+        self, speed=2, smoothPoints=5, collisionShrink=0.05, pathDelay=1
+    ):
         """
         Get paths in format that jaeger expects.  No checking is done, so
-        whoever calls this should check things and decide what to do next.
-
-        Robots must be placed in the desired source orientation, and the
-        desination (lattice) alpha/betas must be specified.
+        whoever calls this should check things on the RobotGrid
+        and decide what to do next.
 
         Parameters
         -----------
         speed: float
-            RPM at output, how fast robots move
+            RPM at output, how fast robots move, max speed is 3
+        smoothPoints: int
+            window width for smoothing a path's velocity profile,
+            in units of steps.  Smooths out fast switching between forward
+            and reverse moves on the axes.
+        collisionShrink: float
+            mm, how much to decrease the collision buffer to allow
+            for smoothing
+        pathDely: float
+            seconds, how far in the future to put the first point, this
+            allows a robot to "catch up" to the expected starting point
+            for the path, if it's not there already.
 
         Returns
         ---------
-        forwardPath: dict
-            alpha/beta points for all robots, moving them from lattice state
-            to target state (aligned with sources).  The out path.
+        toDestination: dict
+            alpha/beta points in time for all robots.  Path begins at robot
+            grid's initialized state, moving toward destination state.
 
-        reversePath: dict
-            alpha/beta points for all robots, moving them from target state
-            to lattice state.  The back path.
+        fromDestination: dict
+            alpha/beta points in time for all robots.  Path begins at robot
+            grid's destination state, moving toward the initialized state.
+            This is just simply a reversed version of toDestination.
         """
         # break out these parameters later
-        smoothPoints = 5
-        collisionShrink = 0.05 # mm
+        # smoothPoints = 5
+        # collisionShrink = 0.05  # mm, this value shoudld probably be a function of stepsize
         # speed = 2 # rpm at output
-        pathDelay = 1 # seconds
+        # pathDelay = 1  # seconds
         ###########
 
-        cb = self.collisionBuffer
-        # self.pathGenGreedy()
-        # self.pathGenMDP(0.8, 0.2)
         self.smoothPaths(smoothPoints)
         self.simplifyPaths()
-        self.setCollisionBuffer(cb - collisionShrink)
+        self.shrinkCollisionBuffer(collisionShrink)
         self.verifySmoothed()
-        self.setCollisionBuffer(cb)
+        self.growCollisionBuffer(collisionShrink)
 
-        forwardPath = {}
-        reversePath = {}
+        toDestination = {}
+        fromDestination = {}
 
         for r in self.robotDict.values():
 
@@ -211,16 +224,11 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
             if r.isOffline:
                 continue
 
-            # bp = numpy.array(r.betaPath)
-            # sbp = numpy.array(r.interpSmoothBetaPath)
             ibp = np.array(r.simplifiedBetaPath)
 
-            # ap = numpy.array(r.alphaPath)
-            # sap = numpy.array(r.interpSmoothAlphaPath)
+
             iap = np.array(r.simplifiedAlphaPath)
 
-            # generate kaiju trajectory (for robot 23)
-            # time = angStep * stepNum / speed
 
             alphaTimesR = iap[:, 0] * self.stepSize / (speed * 360 / 60.)
             alphaDegR = iap[:, 1]
@@ -230,13 +238,14 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
 
             # add time buffer for the reverse path, in case robot is
             # not exactly starting from the expected spot.
-            armPathR = {}  # reverse path
+            # build path from initial state to destination
+            armPathR = {}
             armPathR["alpha"] = [(pos, time + pathDelay) for pos, time in zip(alphaDegR, alphaTimesR)]
             armPathR["beta"] = [(pos, time + pathDelay) for pos, time in zip(betaDegR, betaTimesR)]
 
-            reversePath[int(r.id)] = armPathR
+            toDestination[int(r.id)] = armPathR
 
-            # build forward path
+            # build path from destination to initial state
             alphaTimesF = np.abs(alphaTimesR - alphaTimesR[-1])[::-1]
             alphaDegF = alphaDegR[::-1]
             betaTimesF = np.abs(betaTimesR - betaTimesR[-1])[::-1]
@@ -247,9 +256,9 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
             armPathF["beta"] = [(pos, time + pathDelay) for pos, time in zip(betaDegF, betaTimesF)]
 
 
-            forwardPath[int(r.id)] = armPathF
+            fromDestination[int(r.id)] = armPathF
 
-        return forwardPath, reversePath
+        return toDestination, fromDestination
 
     def addRobot(self,
         robotID, holeID, basePos, hasApogee=True,
@@ -259,7 +268,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         metBetaXY=coordio.defaults.MET_BETA_XY,
         bossBetaXY=coordio.defaults.BOSS_BETA_XY,
         apBetaXY=coordio.defaults.AP_BETA_XY,
-        collisionSegBetaXY=None
+        collisionSegBetaXY=None, collisionBuffer=2, lefthanded=False
     ):
 
         if collisionSegBetaXY is None:
@@ -271,9 +280,8 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
             # how much space around tip to save, set to zero
             # to allow "corner brushes"?
             # cornerProtection = 0.3  # mm
-            cornerProtection = 0.4
-            r = self.collisionBuffer
-            dxTip = np.sqrt(r**2 + (tipFlat/2)**2) - cornerProtection
+            cornerProtection = 0.55
+            dxTip = np.sqrt(collisionBuffer**2 + (tipFlat/2)**2) - cornerProtection
             #dxPost = r - betaPost
             dxPost = 0
             collisionSegBetaXY = [
@@ -290,7 +298,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
             list(kHat), list(dxyz), alphaLen, alphaOffDeg,
             betaOffDeg, elementHeight, self.scaleFac, list(metBetaXY),
             list(bossBetaXY), list(apBetaXY), collisionSegBetaXY,
-            hasApogee
+            hasApogee, collisionBuffer, lefthanded
         )
 
     def addFiducial(self, fiducialID, xyzWok, collisionBuffer=1.5):
@@ -355,7 +363,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         Dark blue are assigned robots.
         Red are collided robots
         Light blue are unassigned robots
-"""
+        """
         plt.figure(figsize=(10, 10))
         ax = plt.gca()
 
@@ -368,7 +376,6 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         plt.ylim(np.array([-1., 1.]) * rr)
         return
 
-
     def tofits(self, filename=None):
         """Write robot information to FITS file
 
@@ -383,7 +390,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
 
         HDU1 contains the robot information
         HDU2 contains the target information
-"""
+        """
         robot_array = self.robot_array()
         target_array = self.target_array()
         fitsio.write(filename, robot_array, clobber=True)
@@ -403,7 +410,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         --------
 
         Assumes the format written out by tofits()
-"""
+        """
         robot_array = fitsio.read(filename, ext=1)
         target_array = fitsio.read(filename, ext=2)
         self.clearTargetList()
@@ -592,10 +599,15 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         using __dict__ but haven't figured it out yet
         """
 
+        # get collision buffer from the first robot
+        # no longer lives on the RobotGrid
+        # note robots may have unique collision buffer sizes now...
+        cb = list(self.robotDict.items())[0].collisionBuffer
+
         r = {}
         r["nRobots"] = self.nRobots
         r["epsilon"] = self.epsilon
-        r["collisionBuffer"] = self.collisionBuffer
+        r["collisionBuffer"] = cb
         r["angStep"] = self.angStep
         r["didFail"] = self.didFail
         r["nSteps"] = self.nSteps
@@ -694,7 +706,6 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         else:
             return json.dumps(d)
 
-
     def robot_dict(self):
         """Create dictionary with robot information
 
@@ -703,7 +714,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
 
         Each value in dictionary contains a list with an element per robot.
         Fiducial information is not included.
-"""
+        """
         robot_dict = dict()
         rd = self.robotDict
         ks = rd.keys()
@@ -744,7 +755,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         --------
 
         Each value in dictionary contains a list with an element per target.
-"""
+        """
         ks = list(self.targetDict.keys())
         nks = len(ks)
         target_dict = dict()
@@ -770,7 +781,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
 
         Probably this should just return the jsons and not the "robot_obj ="
         and "target_obj =" parts.
-"""
+        """
         robot_dict = self.robot_dict()
         target_dict = self.target_dict()
         json_str = '{'
@@ -787,7 +798,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
 
         filebase : str
             base of html and json to write to
-"""
+        """
         fieldfile = filebase + '.json'
         fp = open(fieldfile, "w")
         fp.write(self.json())
@@ -893,7 +904,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
 
         Does not clearTargetList() first, so will add targets to
         existing list.
-"""
+        """
         for it in np.arange(len(target_array)):
             ft = target_array['fiberType'][it].decode().strip()
             fiberType = str2FiberType[ft]
@@ -924,7 +935,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
         consistent with the robot_array information. It double
         checks that any information in robot_array is consistent
         with this robotGrid.
-"""
+        """
         for ir, r in enumerate(robot_array):
             if(r['isAssigned']):
                 indx = r['assignedTargetID']
@@ -940,10 +951,7 @@ class RobotGrid(kaiju.cKaiju.RobotGrid):
                     raise RuntimeError("Inconsistency in robot file in column {n}".format(n=n))
         return
 
-
-class RobotGridCalib(RobotGrid):
-
-    def _load_grid(self):
+    def _load_from_tables(self, positionerTable, wokCoords, fiducialCoords):
 
         df = pd.merge(positionerTable, wokCoords, on="holeID").reset_index()
 
@@ -964,114 +972,147 @@ class RobotGridCalib(RobotGrid):
                 metBetaXY=[row.metX, row.metY],
                 bossBetaXY=[row.bossX, row.bossY],
                 apBetaXY=[row.apX, row.apY],
-                collisionSegBetaXY=None
+                collisionSegBetaXY=None,
+                collisionBuffer=2
             )
 
         for ii, row in fiducialCoords.iterrows():
             self.addFiducial(
                 fiducialID=int(row.id.strip("F")),
                 xyzWok=[row.xWok, row.yWok, row.zWok],
-                collisionBuffer=2
+                collisionBuffer=3
             )
 
+        # gfa locations hard-coded for now
+        # eventually read from a table
+        gfaList = [
+            [
+                [-267.49, 89.15, coordio.defaults.POSITIONER_HEIGHT],
+                [-210.95, 187.08, coordio.defaults.POSITIONER_HEIGHT]
+            ],
+            [
+                [-56.54, 276.23, coordio.defaults.POSITIONER_HEIGHT],
+                [56.54, 276.23, coordio.defaults.POSITIONER_HEIGHT]
+            ],
+            [
+                [210.95, 187.07, coordio.defaults.POSITIONER_HEIGHT],
+                [267.49, 89.15, coordio.defaults.POSITIONER_HEIGHT]
+            ],
+            [
+                [267.49, -89.15, coordio.defaults.POSITIONER_HEIGHT],
+                [210.95, -187.08, coordio.defaults.POSITIONER_HEIGHT]
+            ],
+            [
+                [56.54, -276.23, coordio.defaults.POSITIONER_HEIGHT],
+                [-56.54, -276.23, coordio.defaults.POSITIONER_HEIGHT]
+            ],
+            [
+                [-210.95, -187.07, coordio.defaults.POSITIONER_HEIGHT],
+                [-267.49, -89.15, coordio.defaults.POSITIONER_HEIGHT]
+            ]
+        ]
+
+        for ii, collisionSegXYZWok in enumerate(gfaList):
+            self.addGFA(ii, collisionSegXYZWok, collisionBuffer=3)
+
         self.initGrid()
 
+    def plot_state(
+            self, highlightRobot=None, plotRobotIDs=True,
+            returnax=True, figname="kaijuGrid.pdf"
+        ):
+        """Plot the current state of the robot grid
+
+        Unassigned robots are skyblue
+        Robots at their destination are gold
+        Robots that are kaiju-collided are red
+        Offline robots are black
+
+        Parameters
+        ------------
+        highlightRobot : int, list, or None
+            Robot id to highlight in Orange
+
+        plotRobotIDs : bool
+            If True, plot the robotIDs
+
+        returnax : bool
+            If True, return the matplotlib axis (no figure is written to disk)
+
+        figname : str
+            If returnax is False, write the image to disk with this path.
+            Defaults to kaijuGrid.png.
+
+        Returns
+        ---------
+        ax : matplotlib axes instance if returnax is True
+
+        """
+        from kaiju.utils import plotOne
+        if returnax:
+            ax = plotOne(
+                step=0, robotGrid=self, isSequence=False,
+                plotRobotIDs=plotRobotIDs,
+                highlightRobot=highlightRobot, returnax=True
+            )
+            return ax
+        else:
+            plotOne(
+                step=0, robotGrid=self, figname=figname, isSequence=False,
+                plotRobotIDs=plotRobotIDs,
+                highlightRobot=highlightRobot
+            )
+
+
+class RobotGridCalib(RobotGrid):
+
+    def _load_grid(self):
+        self._load_from_tables(
+            calibration.positionerTable,
+            calibration.wokCoords,
+            calibration.fiducialCoords
+        )
+
 
 class RobotGridAPO(RobotGrid):
-    """APO grid class for robots in FPS
-
-    Parameters:
-    ----------
-
-    stepSize : float, np.float32
-        step size for paths (degrees), default 1.
-
-    collisionBuffer : float, np.float32
-        half-width of beta arm, including buffer (mm), default 2.0
-
-    Attributes:
-    ----------
-
-    stepSize : float, np.float32
-        step size for paths (degrees). A maximum perturbation allowed for either
-        alpha or beta axes
-
-    collisionBuffer : float, np.float32
-        half-width of beta arm, including buffer (mm)
-
-    epsilon : float
-        smoothing parameter used in the RDP path simplification.  Smaller values
-        mean smaller deviations from the rough path are allowed.
-
-    seed : int
-        seed for random number generator when used
-
-    robotDict : dictionary of Robot class objects
-        all robots
-
-    nRobots : int
-        number of robots
-
-    fiducialDict : dictionary of Fiducial objects
-        positions of fiducials
-
-    targetDict : dictionary of Target class objects
-        targets in field, with ID as keys
-
-    smoothCollisions : int
-        number of collisions detected after attempted path smoothing, should
-        be zero when things work out
-
-    didFail : bool
-        path generation failed, not all robots reached target
-
-    nSteps : int
-        steps taken to
-
-"""
 
     def _load_grid(self):
-        """Load filled hex grid of robot locations"""
-        gridFile = os.path.join(KAIJU_ETC_DIR, "fps_DesignReference.txt")
+
+        self._load_from_tables(
+            calibration.positionerTable.loc["APO"],
+            calibration.wokCoords.loc["APO"],
+            calibration.fiducialCoords.loc["APO"]
+        )
 
 
-        robotID = 1
-        fiducialID = 1
-        with open(gridFile, "r") as f:
-            lines = f.readlines()
-        for line in lines:
-            line = line.strip()
-            line = line.split("#")[0]
-            if not line:
-                continue
-            row, col, x, y, fType = line.split()
-            # make col 1 indexed to match pdf maps
-            # and wok hole naming convention
-            col = "C" + str(int(col) + 1)
-            if row == "0" or row.startswith("-"):
-                row = "R" + row
-            else:
-                row = "R+" + row
+class RobotGridLCO(RobotGrid):
 
-            holeName = row + col
+    def _load_grid(self):
+        self._load_from_tables(
+            calibration.positionerTable.loc["LCO"],
+            calibration.wokCoords.loc["LCO"],
+            calibration.fiducialCoords.loc["LCO"]
+        )
 
-            if fType == "BA":
-                self.addRobot(robotID, holeName, [float(x), float(y), 0], hasApogee=True)
-                robotID += 1
-            elif fType == "BOSS":
-                self.addRobot(robotID, holeName, [float(x), float(y), 0], hasApogee=False)
-                robotID += 1
-            elif fType == "Fiducial":
-                self.addFiducial(fiducialID, [float(x), float(y), coordio.defaults.POSITIONER_HEIGHT], self.collisionBuffer)
-                fiducialID += 1
-            else:
-                # ignore other elements (like empty holes)
-                pass
 
-        self.initGrid()
+class RobotGridNominal(RobotGrid):
+    # constructed from a "perfect" grid
+    # useful for testing because calib grid will change overtime
+    # this is a flat wok model, with GFAs
+    def _load_grid(self):
 
-        return()
+        positionerTable = pd.read_csv(
+            os.path.join(KAIJU_ETC_DIR, "positionerTable.csv")
+        )
+
+        wokCoords = pd.read_csv(
+            os.path.join(KAIJU_ETC_DIR, "wokCoords.csv")
+        )
+
+        fiducialCoords = pd.read_csv(
+            os.path.join(KAIJU_ETC_DIR, "fiducialCoords.csv")
+        )
+
+        self._load_from_tables(positionerTable, wokCoords, fiducialCoords)
 
 
-class RobotGridLCO(RobotGridAPO):
-    pass
